/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    ArmarXSimulation::gui-plugins::ClutteredSceneGeneratorWidgetController
 * \author     Patrick Hegemann ( patrick dot hegemann at kit dot edu )
 * \date       2021
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "ClutteredSceneGeneratorWidgetController.h"

#include <QListWidget>
#include <QFlags>

#include <boost/algorithm/string/predicate.hpp>

#include <glob.h>

#include <ArmarXSimulation/libraries/ClutteredSceneGenerator/ClutteredSceneGenerator.h>



namespace armarx
{
    ClutteredSceneGeneratorWidgetController::ClutteredSceneGeneratorWidgetController()
    {
        widget.setupUi(getWidget());

        // TODO: Configuration dialog for simulator proxy

        // TODO: Set default values for object directories
        //        const string ClutteredGraspingScene_DATA_DIR = "";
        //        fs::path objectPath(ClutteredGraspingScene_DATA_DIR);
        //        objectPath /= "ClutteredGraspingScene/objects";
        //        QString pathText(objectPath.c_str());
        //        widget.lineEditObjectsOutputPath->setText(pathText);
        //        widget.lineEditObjectsInputPath->setText(pathText);
        //        refreshSets();
        //        presetStandardColors();

        updateObjectPoseConstraints();
    }


    ClutteredSceneGeneratorWidgetController::~ClutteredSceneGeneratorWidgetController()
    {

    }


    void ClutteredSceneGeneratorWidgetController::loadSettings(QSettings* settings)
    {

    }

    void ClutteredSceneGeneratorWidgetController::saveSettings(QSettings* settings)
    {

    }


    void ClutteredSceneGeneratorWidgetController::onInitComponent()
    {
        usingProxy(simulatorProxyName);
    }


    void ClutteredSceneGeneratorWidgetController::onConnectComponent()
    {
        // Scene generation tab
        connect(widget.pushButtonRefreshObjectSets, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::refreshSets);
//        connect(widget.pushButtonPresetStandardColors, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::presetStandardColors);
//        connect(widget.pushButtonPresetRandomColors, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::presetRandomColors);
//        connect(widget.pushButtonPresetAllSets, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::presetSelectAllSets);
//        connect(widget.pushButtonPresetDeselectAll, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::presetDeselectAllSets);
        connect(widget.pushButtonGenerateNewScene, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::generateScene);
//        connect(widget.pushButtonGenerateSceneBatch, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::generateSceneBatch);
//        connect(widget.pushButtonDropObjects, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::dropObjects);
//        connect(widget.pushButtonResetInvalidObjects, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::resetInvalidObjects);

        connect(widget.checkBoxAutoPoseConstraints, &QCheckBox::stateChanged, this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);
        connect(widget.spinBoxTableX, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);
        connect(widget.spinBoxTableY, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);
        connect(widget.spinBoxTableZ, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);
        connect(widget.spinBoxTableW, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);
        connect(widget.spinBoxTableH, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);
        connect(widget.spinBoxTableD, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints);

        // Objects tab
//        connect(widget.pushButtonGenerateObjects, &QPushButton::clicked, this, &ClutteredSceneGeneratorWidgetController::generateObjects);

        simulator = getProxy<SimulatorInterfacePrx>(simulatorProxyName);
    }

    void ClutteredSceneGeneratorWidgetController::generateObjects() {

    }

    void ClutteredSceneGeneratorWidgetController::refreshSets() {
        path path(widget.lineEditObjectsInputPath->text().toStdString());
        objectSets = getObjectSets(path);

        // Clear list and delete all old items
        widget.listWidgetObjectSets->clear();
        objectSetListItems.clear();
        for (auto& item : objectSetListItems)
        {
            delete item;
        }

        // Fill list widget
        for (const auto& [setName, _] : objectSets)
        {
            auto* item = new QListWidgetItem;
            objectSetListItems.push_back(item);

            // TODO: Is there a safer method to get the name of a directory?
            string basename = setName.substr(setName.find_last_of("/") + 1);
            item->setText(QString::fromStdString(basename));
            item->setData(1, QString::fromStdString(setName));
            item->setFlags((item->flags() | Qt::ItemIsUserCheckable) & ~Qt::ItemIsSelectable);
            item->setCheckState(Qt::Unchecked);
            widget.listWidgetObjectSets->addItem(item);
        }
    }

//    void ClutteredSceneGeneratorWidgetController::presetStandardColors() {
//
//    }
//
//    void ClutteredSceneGeneratorWidgetController::presetRandomColors() {
//
//    }
//
//    void ClutteredSceneGeneratorWidgetController::presetSelectAllSets() {
//
//    }
//
//    void ClutteredSceneGeneratorWidgetController::presetDeselectAllSets() {
//
//    }

    void ClutteredSceneGeneratorWidgetController::updateObjectPoseConstraints() {
        if (widget.checkBoxAutoPoseConstraints->isChecked()) {
            const double tableX = widget.spinBoxTableX->value();
            const double tableY = widget.spinBoxTableY->value();
            const double tableZ = widget.spinBoxTableZ->value();
            const double tableW = widget.spinBoxTableW->value();
            const double tableH = widget.spinBoxTableH->value();
            const double tableD = widget.spinBoxTableD->value();
            const double margin = widget.spinBoxMarginBoxWalls->value();

            const double minX = tableX - tableW / 2 + margin;
            const double minY = tableY - tableH / 2 + margin;
            const double maxX = tableX + tableW / 2 - margin;
            const double maxY = tableY + tableH / 2 - margin;
            const double minZ = tableZ + tableD + 150;

            widget.spinBoxObjectMinX->setValue(minX);
            widget.spinBoxObjectMinY->setValue(minY);
            widget.spinBoxObjectMaxX->setValue(maxX);
            widget.spinBoxObjectMaxY->setValue(maxY);
            widget.spinBoxObjectMinZ->setValue(minZ);
        }
    }

    void ClutteredSceneGeneratorWidgetController::generateScene() {
        ClutteredSceneGenerator generator(simulator, getGenConfig());
        int seed = widget.spinBoxSceneSeed->value();
        generator.generateScene(seed);
    }

    map<string, vector<string>> ClutteredSceneGeneratorWidgetController::getObjectSets(const path& directory) {
        map<string, vector<string>> sets;

        // Find all sets
        path setPaths = directory / "set_*";
        glob_t globSets;
        glob(setPaths.c_str(), GLOB_TILDE | GLOB_ONLYDIR, nullptr, &globSets);

        // Add all sets
        for (size_t i = 0; i < globSets.gl_pathc; ++i)
        {
            string setPath(globSets.gl_pathv[i]);

            // Find all objects in set
            path objectsPath(setPath);
            objectsPath /= "object*.xml";
            glob_t globObjects;
            glob(objectsPath.c_str(), GLOB_TILDE, nullptr, &globObjects);
            vector<string> objects(globObjects.gl_pathv, globObjects.gl_pathv + globObjects.gl_pathc);

            sets[setPath] = objects;
        }

        return sets;
    }

    ClutteredSceneGenerator::Config ClutteredSceneGeneratorWidgetController::getGenConfig() {
        ClutteredSceneGenerator::Config genConfig;

        // Set selected object sets
        vector<vector<string>> selectedSets;
        for (int i = 0; i < widget.listWidgetObjectSets->count(); ++i)
        {
            auto* item = widget.listWidgetObjectSets->item(i);
            if (item->checkState() == Qt::CheckState::Checked)
            {
                string setPath = item->data(1).toString().toStdString();
                selectedSets.push_back(objectSets[setPath]);
            }
        }
        genConfig.objectSetFiles = selectedSets;

        genConfig.amountObjects = widget.spinBoxSceneObjectAmount->value();
        genConfig.objectsPerSetLimit = widget.checkBoxLimitOnePerSet->isChecked() ? 1 : 0;
        genConfig.fallingSteps = widget.spinBoxObjectFallingSteps->value();
        genConfig.autoResetInvalidObjects = widget.checkBoxResetInvalidObjects->isChecked();
        genConfig.maxResetTries = widget.spinBoxMaxResetAttempts->value();

        genConfig.boxX = widget.spinBoxTableX->value();
        genConfig.boxY = widget.spinBoxTableY->value();
        genConfig.boxZ = widget.spinBoxTableZ->value();
        genConfig.boxW = widget.spinBoxTableW->value();
        genConfig.boxH = widget.spinBoxTableH->value();
        genConfig.boxD = widget.spinBoxTableD->value();

        genConfig.objectMarginSide = widget.spinBoxMarginBoxWalls->value();
        genConfig.objectSpacingZ = widget.spinBoxObjectSpacingZ->value();

        genConfig.minObjectX = widget.spinBoxObjectMinX->value();
        genConfig.minObjectY = widget.spinBoxObjectMinY->value();
        genConfig.minObjectZ = widget.spinBoxObjectMinZ->value();
        genConfig.maxObjectX = widget.spinBoxObjectMaxX->value();
        genConfig.maxObjectY = widget.spinBoxObjectMaxY->value();

        genConfig.minObjectZOtherwiseReset = widget.spinBoxZPosResetting->value();

        return genConfig;
    }


}
